#include "buffer.h"
#include "vi92.h"
#include "plottxt.h"

tHandle BufferSave(tBuffer * Buffer)
{
 tHandle Handle;
 size_t size = 0;
 tLigne * LigneCourante;
 char * Fichier;

 LigneCourante = Buffer->PremiereLigne;
 while(LigneCourante)
 {
  size += LigneCourante->Longueur + 2; // +2 pour le '\n' et le ' '
  LigneCourante = LigneCourante->LigneSuivante;
 }

 size += 6; // taille + curseur + 0x00E0(tag)
 Handle = HeapAlloc(size);

 Fichier = HeapDeref(Handle);


 *((int *)Fichier) = size - 3; // Taille
 Fichier += 2;
 *(Fichier++) = 0; // Curseur
 *(Fichier++) = 1;

 LigneCourante = Buffer->PremiereLigne;
 while(LigneCourante)
 {
    *(Fichier++) = ' ';
    strcpy(Fichier, LigneCourante->Texte);
    Fichier += LigneCourante->Longueur;
    *(Fichier++) = 13;
    LigneCourante = LigneCourante->LigneSuivante;
 }

 Fichier--; // car on efface le dernier saut de ligne !
 *(Fichier++) = 0; // Tag 
 *(Fichier++) = 0xE0;

 return Handle; 
}

tLigne * BufferLigne(tBuffer * Buffer, int NumLigne)
{
 tLigne * LigneCourante;
 int NumLigneCourante = 1;

 LigneCourante = Buffer->PremiereLigne;

 while(NumLigneCourante < NumLigne)
 {
  LigneCourante = LigneCourante->LigneSuivante;
  NumLigneCourante++;
 }

 return LigneCourante;
}


tBuffer * BufferLoad(char * Texte)
{
 char * Next;
 int Len;
 tLigne * LigneCourante;
 tBuffer * Buffer = (tBuffer *)malloc(sizeof(tBuffer));

 Buffer->NbLignes = 0;
 Buffer->NumLigneCourante = 1;
 Buffer->PremiereLigne = (tLigne *)malloc(sizeof(tLigne));
 LigneCourante = Buffer->PremiereLigne;
 LigneCourante->LigneSuivante = NULL;
 LigneCourante->LignePrecedente = NULL;
 LigneCourante->Texte = NULL;


 while(Next = strchr(Texte, 13))
 {
  Texte++; // Pour supprimer le ' '
  Next;
  Len = Next - Texte;

  Buffer->NbLignes++;

  LigneCourante->Texte = (char *)malloc(Len + 1);
  strncpy(LigneCourante->Texte, Texte, Len);
  LigneCourante->Texte[Len] = 0;
  LigneCourante->Longueur = Len;
  LigneCourante->Hauteur = (Len - 1) / 40 + 1;

  LigneCourante->LigneSuivante = (tLigne *)malloc(sizeof(tLigne));
  LigneCourante->LigneSuivante->LignePrecedente = LigneCourante;
  LigneCourante = LigneCourante->LigneSuivante;
  LigneCourante->LigneSuivante = NULL;
  LigneCourante->Texte = NULL;

  Texte = Next + 1; // pour passer le '\n'
 }

 if(*Texte != 0)
 {
  Texte++; // Pour supprimer le ' '
  Buffer->NbLignes++;

  Len = strlen(Texte);
  LigneCourante->Texte = (char *)malloc(Len + 1);
  strncpy(LigneCourante->Texte, Texte, Len);
  LigneCourante->Texte[Len] = 0;
  LigneCourante->Longueur = Len;
  LigneCourante->Hauteur = (Len - 1) / 40 + 1;
 }
 else
 {
  LigneCourante = LigneCourante->LignePrecedente;
  free(LigneCourante->LigneSuivante);
 }

 Buffer->Modified = 0;

 return Buffer;
}

void BufferFree(tBuffer * Buffer)
{
 tLigne * LigneCourante = Buffer->PremiereLigne;
 while(LigneCourante->LigneSuivante) LigneCourante = LigneCourante->LigneSuivante;

 while(LigneCourante->LignePrecedente)
 {
  LigneCourante = LigneCourante->LignePrecedente;
  free(LigneCourante->LigneSuivante->Texte);
  free(LigneCourante->LigneSuivante);
 }

 free(Buffer->PremiereLigne);
 free(Buffer);
}


void BufferDisplay(tBuffer * Buffer)
{
 int LigneLCD = 0, Len;
 tLigne * LigneCourante;
 char * Texte, * pointer, * pointer2;
 int NumLigneCourante = Buffer->NumLigneCourante;

 clr_DoubleBuffer();

 LigneCourante = BufferLigne(Buffer, NumLigneCourante);

 while(LigneCourante && (LigneLCD <= 15))
 {
  Len = LigneCourante->Longueur;
  if(!((ExtraChar && (NumLigneCourante == CursorLine)) || (Len == 0)))
  {
     Texte = LigneCourante->Texte;
     pointer = NULL;
  }
  else
  {
     Texte = (char *)malloc(Len + 2);
     strcpy(Texte, LigneCourante->Texte);
     Texte[Len] = ' ';
     Len++;
     Texte[Len] = '\0';
     pointer = Texte;
  }

  pointer2 = NULL;
  if(DispLineNumber)
  {
     pointer2 = (char *)malloc(Len + 6);
     sprintf(pointer2, "% 4d ", NumLigneCourante);
     strcpy(pointer2 + 5, Texte);
     Len += 5;
     Texte = pointer2;
  }

  while((Len > 0) && (LigneLCD <= 15))
  {
   Len -= PlotText(Texte, LigneLCD++);
   Texte += 40;
  }

  if(pointer2) free(pointer2);
  if(pointer) free(pointer);
  NumLigneCourante++;
  LigneCourante = LigneCourante->LigneSuivante;
 }

 DisplayDoubleBuffer();
}


void LigneInsererCaractere(tLigne * Ligne, int Position, unsigned char Caractere)
{
 int Len = Ligne->Longueur;
 char * Texte = (char *)malloc(Len + 2);
 char * OrigTexte = Ligne->Texte;

 if(Len)
 {
    strncpy(Texte, OrigTexte, Position - 1);
    Texte[Position - 1] = Caractere;
    strcpy(Texte + Position, OrigTexte + Position - 1);
 }
 else
 {
    Texte[0] = Caractere;
 }
 Texte[Len + 1] = 0;

 Ligne->Texte = Texte;
 Ligne->Longueur++;
 Ligne->Hauteur = (Len) / 40 + 1;

 free(OrigTexte);
 Buffer->Modified = 1;
}

void LigneSupprimerCaractere(tLigne * Ligne, int Position)
{
 int Len = Ligne->Longueur;
 char * Texte;
 char * OrigTexte = Ligne->Texte;

 if(Len > 0)
 {
    Texte = (char *)malloc(Len);
    strncpy(Texte, OrigTexte, Position - 1);
    strcpy(Texte + Position - 1, OrigTexte + Position);
    Texte[Len - 1] = 0;

    Ligne->Texte = Texte;
    Ligne->Longueur--;
    Ligne->Hauteur = (Len - 2) / 40 + 1;

    free(OrigTexte);
    Buffer->Modified = 1;
 }
}

void BufferInsererLigne(tBuffer * Buffer, int NumLigne) // Insere apres
{
 tLigne * LigneCourante, * LigneSuivante;
 int NumLigneCourante = 1;

 if(NumLigne > 0)
 {
    LigneCourante = Buffer->PremiereLigne;

    while(NumLigneCourante < NumLigne)
    {
     LigneCourante = LigneCourante->LigneSuivante;
     NumLigneCourante++;
    }

    LigneSuivante = LigneCourante->LigneSuivante;

    LigneCourante->LigneSuivante = (tLigne *)malloc(sizeof(tLigne));
    LigneCourante->LigneSuivante->LignePrecedente = LigneCourante;
    LigneCourante = LigneCourante->LigneSuivante;

    LigneCourante->LigneSuivante = LigneSuivante;
    if(LigneSuivante != NULL)
       LigneCourante->LigneSuivante->LignePrecedente = LigneCourante;

    LigneCourante->Texte = (char *)malloc(1);
    LigneCourante->Texte[0] = 0;

    LigneCourante->Longueur = 0;
    LigneCourante->Hauteur = 1;
    Buffer->Modified = 1;
 }
 else if(NumLigne == 0)
 {
    LigneSuivante = Buffer->PremiereLigne;
    LigneCourante = (tLigne *)malloc(sizeof(tLigne));
    LigneCourante->LigneSuivante = LigneSuivante;
    LigneCourante->LignePrecedente = NULL;
    if(LigneSuivante != NULL)
       LigneCourante->LigneSuivante->LignePrecedente = LigneCourante;


    LigneCourante->Texte = (char *)malloc(1);
    LigneCourante->Texte[0] = 0;

    LigneCourante->Longueur = 0;
    LigneCourante->Hauteur = 1;

    Buffer->PremiereLigne = LigneCourante;
    Buffer->Modified = 1;
 }

 Buffer->NbLignes++;
}

void BufferSupprimerLigne(tBuffer * Buffer, int NumLigne)
{
 tLigne * LigneCourante, * LigneSuivante, * LignePrecedente;
 int NumLigneCourante = 1;

 if(Buffer->NbLignes == 1) return;
 if(NumLigne != 1)
 {
    LigneCourante = Buffer->PremiereLigne;

    while(NumLigneCourante < NumLigne)
    {
     LigneCourante = LigneCourante->LigneSuivante;
     NumLigneCourante++;
    }

    LigneSuivante = LigneCourante->LigneSuivante;
    LignePrecedente = LigneCourante->LignePrecedente;

    if(LigneSuivante)
       LigneSuivante->LignePrecedente = LignePrecedente;
    LignePrecedente->LigneSuivante = LigneSuivante;

    free(LigneCourante->Texte);
    free(LigneCourante);
    Buffer->Modified = 1;
 }
 else
 {
    LigneCourante = Buffer->PremiereLigne;
    LigneSuivante = LigneCourante->LigneSuivante;
    Buffer->PremiereLigne = LigneSuivante;
    LigneSuivante->LignePrecedente = NULL;
    free(LigneCourante->Texte);
    free(LigneCourante);
    Buffer->Modified = 1;
 }

 Buffer->NbLignes--;
}

void LigneRemplacerTexte(tLigne * Ligne, char * Texte)
{
 int Len = strlen(Texte);

 free(Ligne->Texte);

 Ligne->Texte = (char *)malloc(Len + 1);
 strcpy(Ligne->Texte, Texte);
 Ligne->Longueur = Len;
 Ligne->Hauteur = (Len - 1) / 40 + 1;
 Buffer->Modified = 1;
}

void LigneTronquer(tLigne * Ligne, int Len)
{
 char * Texte = (char *)malloc(Len + 1);

 strncpy(Texte, Ligne->Texte, Len);
 Texte[Len] = 0;

 free(Ligne->Texte);
 Ligne->Texte = Texte;

 Ligne->Longueur = Len;
 Ligne->Hauteur = (Len - 1) / 40 + 1;
 Buffer->Modified = 1;
}

void BufferAjusteHauteur(tBuffer * Buffer)
{
 tLigne * LigneCourante = Buffer->PremiereLigne;
 int Len;

 while(LigneCourante)
 {
    Len = strlen(LigneCourante->Texte);
    LigneCourante->Longueur = Len;
    Len  += (DispLineNumber?5:0);
    LigneCourante->Hauteur = (Len - 1) / 40 + 1;
    LigneCourante = LigneCourante->LigneSuivante;
 }
}

void LigneStrCat(tLigne * Ligne, char * str)
{
 int Len;
 char * Texte;

 Len = Ligne->Longueur + strlen(str);
 Texte = (char *)malloc(Len + 1);

 strcpy(Texte, Ligne->Texte);
 strcat(Texte, str);
 Ligne->Longueur = Len;
 Ligne->Hauteur = (Len - 1) / 40 + 1;
 free(Ligne->Texte);
 Ligne->Texte = Texte;
 Buffer->Modified = 1;
}